# Hua - A PowerShell Static Content Generator

# $Author: jeffr $
# $Date: 2024-04-18 15:56:38 -0400 (Thu, 18 Apr 2024) $
# $Revision: 149 $

#
# Get file and config variables
#

# Show version info and quit
if ($args[0] -eq '-V' -or $args[0] -eq '--version') {
	Write-Host 'Hua version: $Revision: 149 $'
	exit 0
}

# Config file passed as first argument
if ($args[0]) {
	# Make sure the config file exists
	if (Test-Path $args[0] -PathType Leaf) {
		# Config file exists
		$configs = Get-Content -Path $args[0]
		foreach ($config in $configs) {
			if ($config -match '^*=') {
				# Set config variables
				$var_val = $config.split('=')
				$var_val = $var_val.replace(' ', '')
				New-Variable -Name $var_val[0] -Value $var_val[1]
			}
		}
	} else {
		# Config file doesn't exist
		Write-Host 'Config file error. Config file path specified as' $args[0] '. Quitting.'
		exit 1
	}
} else {
	# No config file specified
	Write-Host 'Usage: .\hua.ps1 [path to config file] [optional: -O|--orphans]'
	exit 1
}

# Check to make sure all file/config vars are defined or quit
$config_vars = 'entries_file', 'meta_file', 'content_dir', 'markdown_dir', 'output_dir', 'index_file', 'archive_file', 'header_file', 'footer_file', 'read_more_file', 'comments_file', 'blog_root', 'web_root', 'title_sep', 'entries_pp'
foreach ($config_var in $config_vars) {
	if (-Not (Test-Path variable:local:$config_var)) {
		Write-Host "File/config variable error. $config_var variable is not defined. Quitting."
		exit 1
	}
}

#
# ProcessTags
#
# 	input:	an array of tags
#	output: an html list of unique sorted tags
#
function ProcessTags {
	$tags = $args[0] | Sort-Object -Unique
	foreach ($tag in $tags) {
		$tag_link = $tag.replace(' ', '-')
		$tags_list += "<li><a href=`"tagged-with-$tag_link.html`"><span class=`"tag`">$tag</span></a></li>"
	}
	$tags_list
}

#
# ListOrphans
#
# 	input:	contents of entries_file, an array of output_dir article filenames
#	output: a list of orphans to stdout
#
function ListOrphans {
	foreach ($out_path in $args[1]) {
		$out_file = Split-Path $out_path -leaf
		$out_file_no_ext = [System.IO.Path]::GetFileNameWithoutExtension($out_file) + '.md'

		# If the output file is not represented in the entries file, it may be an orphan
		if (-Not ($args[0].'File'.contains($out_file)) -And (-Not ($args[0].'File'.contains($out_file_no_ext))) -And (-Not ($out_file -match 'tagged-with-*'))) {
			Write-Host $out_file
		}
	}
}

#
# MasterTags
#
# 	input: a file handle, tag list string, scope (1 or all)
#	output: addition/removal of master tag list in output HTML
#
function MasterTags {
	$fh = $args[0]
	$tl = $args[1]
	$sc = $args[2]

	if (Test-Path $fh -PathType Leaf) {
		$fh_content = Get-Content -Raw $fh
	} else {
		continue
	}

	# $fh_content = Get-Content -Raw $fh
	[regex]$pattern = "--master_tags--"

	# If no --master_tags--, processing has already occurred on file so skip
	if (-Not ($fh_content -match $pattern)) {
		continue
	}

	if ($sc -eq 1) {
		# Replace the first
		$fh_content = $pattern.replace($fh_content, '<ul class="tags">' + $tl + '</ul>', 1)
	} else {
		# Replace all
		$fh_content = $pattern.replace($fh_content, '<ul class="tags">' + $tl + '</ul>')
	}

	# Write out the new file
	Set-Content $fh $fh_content
}

#
# IndexFileLup
#
# 	input: Entry ID
#	output: Filename of index page on which it appears
#
function IndexFileLup {
	$id = $args[0]
	$entries_l = Import-Csv -Path 'paging.csv' -Delimiter ","
	$page = $entries_l | Where-Object {$_.'ID' -eq $id} | ForEach-Object Page
	return $page
}

#
# TaggedWithLup
#
# 	input: Entry ID and tag
#	output: Filename of tagged with page on which it appears
#
function TaggedWithLup {
	$id = $args[0]
	$tag = $args[1]
	$entries_l = Import-Csv -Path 'paging.csv' -Delimiter ","

	$tags_l = $entries_l | Where-Object {$_.'ID' -eq $id} | ForEach-Object 'Tags'
	$tw_l = $entries_l | Where-Object {$_.'ID' -eq $id} | ForEach-Object 'TW Page'

	$tags_l = $tags_l -split '[|]'
	$tw_l = $tw_l -split '[|]'

	# Get position of tag
	$tag_i = $tags_l.IndexOf($tag)
	return $tw_l[$tag_i]
}

#
# PagingUi
#
# 	input: Page filename
#	output: Links to previous and next pages
#
function PagingUi {
	# Return nothing if index is single page
	$entries_l = Import-Csv -Path 'paging.csv' -Delimiter ","
	$page_f = 0
	foreach ($entry_t in $entries_l) {
		if ($entry_t.'Page' -like "*-2*") {
			$page_f = 1
		}
	}
	if ($page_f -eq 0) {
		return
	}
	
	# Get page number
	$index_base = [io.path]::GetFileNameWithoutExtension($page_old)
	$index_file_base = [io.path]::GetFileNameWithoutExtension($index_file)
	$pn = $index_base.replace("$index_file_base-", '')

	if ($pn -eq $index_base) {
		# First page
		$next_link = '<a href="' + $index_file_base + '-2.html">Next Page</a>'
		return '<p class="hua-paging">' + $next_link + '</p>'
	} else {
		# Page 2 and on
		if ($page -ne $page_old) {
			$pn_p = [int]$pn - 1
			$pn_n = [int]$pn + 1
			# Need exception for previous link to first page
			if ($pn_p -eq 1) {
				$prev_link = '<a href="' + "$index_file_base.html" + '">Previous Page</a>'
				$next_link = '<a href="' + $index_file_base + '-' + $pn_n + '.html">Next Page</a>'
				return '<p class="hua-paging">' + "$prev_link $next_link" + '</p>'
			} else {
				$prev_link = '<a href="' + $index_file_base + '-' + $pn_p + '.html">Previous Page</a>'
				$next_link = '<a href="' + $index_file_base + '-' + $pn_n + '.html">Next Page</a>'
				return '<p class="hua-paging">' + "$prev_link $next_link" + '</p>'
			}
		} else {
			$pn_p = [int]$pn - 1
			$prev_link = '<p class="hua-paging"><a href="' + $index_file_base + '-' + $pn_p + '.html">Previous Page</a></p>'
			return $prev_link
		}
	}
}

# Orphans flag passed as second argument
$orphans_mode = 0
if ($args[1]) {
	if ($args[1] -eq '-O' -or $args[1] -eq '--orphans') {
		$orphans_mode = 1
	} else {
		Write-Host 'Usage: .\hua.ps1 [path to config file] [optional: -O|--orphans]'
		exit 1
	}
}

# Open the entries file and read its contents (or fail)
if (Test-Path $entries_file -PathType Leaf) {
	$entries = Import-Csv -Path $entries_file -Delimiter "," | Sort-Object { [int]($_.ID -replace '\D') } -Descending
} else {
	Write-Host "Entries file error. Entries file path set as $entries_file. Quitting."
	exit 1
}

#
# Check for possible ID, file, and filename conflicts (e.g., entry.html and entry.md)
#

# IDs
$ids = @()
foreach ($entry in $entries) {
	$ids += $entry.ID
}
$ids_u = $ids | Sort-Object | Get-Unique

if ($ids.Length -ne $ids_u.Length) {
	Write-Host 'Entry ID conflict. Check the entries file for repeated IDs. Quitting.'
	exit 1
}

$conflict_f = 0
# Entries file
$fns = @()
foreach ($entry in $entries) {
	if ($entry.'ID' -eq '000000') { continue }
	$fn = [System.IO.Path]::GetFileNameWithoutExtension($entry.File)
	$fns += $fn
}
$fns_u = $fns | Sort-Object | Get-Unique
if ($fns.Length -ne $fns_u.Length) { $conflict_f = 1 }

# Content directory
$fns = @()
$fns_fp = Get-ChildItem -File $content_dir
foreach ($fn in $fns_fp) {
	$fn = $fn.Basename
	$fns += $fn
}
$fns_u = $fns | Sort-Object | Get-Unique
if ($fns.Length -ne $fns_u.Length) { $conflict_f = 1 }

if ($conflict_f -ne 0) {
	Write-Host 'Filename conflict. Check the entries file and content directory. Quitting.'
	Write-Host
	Write-Host 'NOTE: This can happen when an HTML file and a Markdown file have the same basename.'
	Write-Host '      For example: entry.html and entry.md conflict.'
	exit 1
}

# Paging
& "$PSScriptRoot/paging.ps1" $entries_file $entries_pp $index_file $content_dir

# Open the meta info file and read its contents (or fail)
if (Test-Path $meta_file -PathType Leaf) {
	$metas = Import-Csv -Path $meta_file -Delimiter "|"
} else {
	Write-Host "Meta info file error. Meta file path set as $meta_file. Quitting."
	exit 1
}

# Orphans mode? Then get a list of files currently in the output directory
if ($orphans_mode -eq 1) {
	$output_articles = Get-Item(Join-Path -Path $output_dir -ChildPath '*.html')

	# Add archive and index files to entries so they're not counted as orphans
	$entries += [PSCustomObject]@{ File=[System.IO.Path]::GetFileName($archive_file) }

	# Account for all index files
	$entries_l = Import-Csv -Path 'paging.csv' -Delimiter ","
	foreach ($entry_l in $entries_l) {
		$entries += [PSCustomObject]@{ File=[System.IO.Path]::GetFileName($entry_l.'Page') }
	}

	ListOrphans $entries $output_articles
	Remove-Item 'paging.csv'
	exit 0
}

# Make temporary output directory and index/archive files
$output_dir_tmp = (Join-Path -Path $output_dir "temp")
New-Item -ItemType Directory -Path $output_dir_tmp | Out-Null

$index_file_tmp = (Join-Path -Path $output_dir_tmp (Split-Path $index_file -leaf))
$archive_file_tmp = (Join-Path -Path $output_dir_tmp (Split-Path $archive_file -leaf))

# Get the header content (or fail)
if (Test-Path $header_file -PathType Leaf) {
	$header = Get-Content -Path $header_file
} else {
	Write-Host "Header error. Header path set as $header_file. Quitting."
	exit 1
}

# Get the footer content (or fail)
if (Test-Path $footer_file -PathType Leaf) {
	$footer = Get-Content -Path $footer_file
} else {
	Write-Host "Footer error. Footer path set as $footer_file. Quitting."
	exit 1
}

# Get content of read more link include (or error)
if (Test-Path $read_more_file -PathType Leaf) {
	$read_more = Get-Content -Path $read_more_file
} else {
	Write-Host "Read more file error. Read more file path set as $read_more_file. Continuing."
	$read_more = ''
}

# Get content of comments include (or error)
if (Test-Path $comments_file -PathType Leaf) {
	$comments = Get-Content -Path $comments_file
} else {
	Write-Host "Comments file error. Comments file path set as $comments_file. Continuing."
	$comments = ''
}

# Check to see if the output dir exists. Quit if it doesn't
if (-Not (Test-Path $output_dir -PathType Container)) {
	Write-Host "Output directory error. Output directory path set as $output_dir. Quitting."
	exit 1
}

#
# Process Markdown (check to see if MarkdownToHtml is installed and template exists)
#
$md_exists = Test-Path (Join-Path -Path $markdown_dir -ChildPath 'md-template.html') -PathType Leaf
$no_md = 0
if ((Get-Module -ListAvailable -Name 'MarkdownToHTML') -And ($md_exists)) {
	Convert-MarkdownToHTML -Verbose:$false -Path (Join-Path -Path $content_dir -ChildPath '*.md') -SiteDirectory $content_dir -Template $markdown_dir | Out-Null
} 
else {
	$no_md = 1
}

#
# Loop over the entries (skip the first) and output the content
#

# Array for the all tags list
$master_tag_list = @()

# Default meta info
$def_keys = $metas | Where-Object {$_.ID -eq '000000'} | ForEach-Object Keywords
$def_desc = $metas | Where-Object {$_.ID -eq '000000'} | ForEach-Object Description
$def_auth = $entries | Where-Object {$_.ID -eq '000000'} | ForEach-Object Author

# Write the index and archive file headers
$header = $header.replace('--meta_keywords--', $def_keys)
$header = $header.replace('--meta_description--', $def_desc)
$header = $header.replace('--meta_author--', $def_auth)

Add-Content $archive_file_tmp ($header -replace '--article_title--', "$title_sep Archive")
Add-Content $archive_file_tmp "<h2>Available Articles</h2><ol id=`"archive`">"

$page_old = ''

foreach ($entry in $entries) {
	# No entry ID? Warn and skip
	if (-Not ($entry.'ID')) {
		Write-Host 'Entries file error. There is an entry without an ID. Skipping.'
		continue
	}

	# Skip default entry
	if ($entry.'ID' -eq '000000') {
		continue
	}

	$e_id = $entry.'ID'
	
	# No filename? Warn and skip
	if (-Not ($entry.'File')) {
		Write-Host 'Entries file error. There is an entry without a filename. Skipping.'
		continue
	}

	# Content file missing? Warn and skip
	if (-Not (Test-Path (Join-Path -Path $content_dir -ChildPath $entry.'File') -PathType Leaf)) {
		$content_fp = (Join-Path -Path $content_dir -ChildPath $entry.'File')
		Write-Host "Entry content error. Entry content file path set as $content_fp. Skipping."
		continue
	}

	# Handle paging for index file(s)
	$page = IndexFileLup($e_id)
	if ($page -ne $page_old) {
		# Footer for previous page (if not first page)
		if ($page_old -ne '') {
			$footer_ui = PagingUi($page_old)
			$footer_i = $footer.replace('<!-- paging -->', $footer_ui)
			Add-Content $index_file_tmp $footer_i
		}
		# Header for new page
		$index_file_tmp = (Join-Path -Path $output_dir_tmp (Split-Path $page -leaf))
		Add-Content $index_file_tmp ($header -replace '--article_title--', '')
	}

	# Skip markdown files if no MarkdownToHTML module installed
	if ([System.IO.Path]::GetExtension($entry.File) -eq '.md' -and $no_md -eq 1) {
		Write-Host 'MarkdownToHTML module not installed or markdown template not found. Skipping.'
		continue
	}

	# Handle markdown filenames
	if ([System.IO.Path]::GetExtension($entry.File) -eq '.md') {
		$e_fn = [System.IO.Path]::GetFileNameWithoutExtension($entry.File) + '.html'
	} else {
		$e_fn = $entry.'File'
	}

	if ($entry.'Title') { $e_title = $entry.'Title' } else { $e_title = 'Untitled' }    # Article title
	if ($entry.'Tags') { $e_tags = $entry.'Tags'.split('|') } else { $e_tags = @() }    # Article tags
	if ($entry.'Date') { $e_date = $entry.'Date' } else { $e_date = '' }                # Article date
	if ($entry.'Author') { $e_author = $entry.'Author' } else { $e_author = '' }        # Article author
	if ($entry.'Contact') { $e_contact = $entry.'Contact' } else { $e_contact = '' }    # Article contact/email

	# Array for the article tag list
	$article_tag_list = @()

	# Loop over the tags
	foreach ($tag in $e_tags) {
		$article_tag_list += $tag.Trim()
		$master_tag_list += $tag.Trim()
	}

	#
	# Create/append to the index/permalink file content
	#

	# Get the article content in content_dir
	$e_content = Get-Content -Path (Join-Path -Path $content_dir -ChildPath $e_fn)
	
	# Article-specific tags
	$e_tags_p = ProcessTags $e_tags
	$e_content = $e_content.replace('--article_tags--', '<ul class="tags">' + $e_tags_p + '</ul>')
	
	# Article permalink, date, title, author, etc.
	$e_content = $e_content.replace('--permalink_url--', $e_fn)
	$e_content = $e_content.replace('--article_date--', $e_date)
	$e_content = $e_content.replace('--article_date--', $e_date)
	$e_content = $e_content.replace('--article_title--', $e_title)
	$e_content = $e_content.replace('--author--', $e_author)
	$e_content = $e_content.replace('--contact--', $e_contact)
	Add-Content $index_file_tmp $e_content
	
	# Permalink (read more link)
	Add-Content $index_file_tmp $read_more.replace('--link--', $e_fn)
	
	# Create and add an entry to the archive file
	$archive_content = "<li><cite><a href=`"$e_fn`">$e_title</a></cite>, by $e_author (written $e_date).</li>"
	Add-Content $archive_file_tmp $archive_content

	#
	# Create the permalink file in the temporary output directory
	#

	$permalink_path = (Join-Path -Path $output_dir_tmp -ChildPath $e_fn)

	# "Reset" header content so title can be properly substituted
	$header_tmp = Get-Content -Path $header_file
	$header_tmp = $header_tmp.replace('--article_title--', $title_sep + ' ' + $e_title)

	# Meta info lookup
	$keywords = $metas | Where-Object {$_.ID -eq $e_id} | ForEach-Object Keywords
	$description = $metas | Where-Object {$_.ID -eq $e_id} | ForEach-Object Description
	if ($null -eq $keywords -or $keywords -eq '') {
		$header_tmp = $header_tmp.replace('--meta_keywords--', $def_keys)
	} else {
		$header_tmp = $header_tmp.replace('--meta_keywords--', $keywords)
	}

	if ($null -eq $description) {
		$header_tmp = $header_tmp.replace('--meta_description--', $def_desc)
	} else {
		$header_tmp = $header_tmp.replace('--meta_description--', $description)
	}
	$header_tmp = $header_tmp.replace('--meta_author--', $e_author)

	Add-Content $permalink_path $header_tmp
	Add-Content $permalink_path $e_content

	# Substitute values in comments include
	$e_comments = $comments.replace('--this.page.url--', "$web_root$blog_root$e_fn")
	$e_comments = $e_comments.replace('--this.page.identifier--', $e_id)

	Add-Content $permalink_path $e_comments
	Add-Content $permalink_path $footer

	# 
	# Tag specific files
	#

	# For each article tag, add the article content to the "tagged-with" file
	foreach ($article_tag in $article_tag_list) {
		$tagged_with_fn = TaggedWithLup $e_id $article_tag
		
		# $tagged_with_fn = "tagged-with-$article_tag_url.html"
		$tagged_with_path = (Join-Path -Path $output_dir_tmp -ChildPath $tagged_with_fn)
		
		# If the tagged-with file is new, add the header first
		if (-Not (Test-Path $tagged_with_path -PathType Leaf)) {
			# "Reset" header content so title can be properly substituted
			$header_tmp = Get-Content -Path $header_file
			$header_tmp = $header_tmp.replace('--article_title--',  "$title_sep Articles tagged with &quot;$article_tag&quot;")

			# Meta tags
			$header_tmp = $header_tmp.replace('--meta_keywords--', $article_tag)
			$header_tmp = $header_tmp.replace('--meta_description--', "Articles tagged with $article_tag.")
			$header_tmp = $header_tmp.replace('--meta_author--', $def_auth)

			Add-Content $tagged_with_path $header_tmp
		}
		
		# Write the article content to the file
		Add-Content $tagged_with_path $e_content
		Add-Content $tagged_with_path $read_more.replace('--link--', $e_fn)
	}

	$page_old = $page
}

# Paging for last page
$footer_ui = PagingUi($page_old)
$footer_i = $footer.replace('<!-- paging -->', $footer_ui)
Add-Content $index_file_tmp $footer_i

Add-Content $archive_file_tmp "</ol>$footer"

#
# Post-process "tagged-with" files
#

$tw_pre_l = @()
foreach ($tagged_with_file in Get-Item(Join-Path -Path $output_dir_tmp -ChildPath 'tagged-with-*')) {
	# Single page or multiple pages?
	if ($tagged_with_file -match '-[*0-9].html$') {
		# Multiple, get prefix of filename
		$tagged_with_pre = $tagged_with_file -replace '-[*0-9].html$', ''
		$tw_pre_l = $tw_pre_l + $tagged_with_pre
	}
}

$tw_pre_l = $tw_pre_l | Select-Object -Unique
foreach ($tw_pre_l_i in $tw_pre_l) {
	$tw_pre_base = Split-Path -Path $tw_pre_l_i -Leaf
	$tw_files = Get-ChildItem -Path "$output_dir_tmp\$tw_pre_base*" | Sort-Object { [int]($_ -replace '\D') }
	$tw_c = 0
	$tw_files_l = $tw_files.Length # No. of pages

	foreach ($tw_file in $tw_files) {
		$tw_c ++ # Increment page counter
		if ($tw_c -eq 1) {
			# Condition for first page
			$prev_link = ''
			$next_link = '<a href="' + $tw_pre_base + '-2.html">Next Page</a>'
		} elseif ($tw_c -lt $tw_files_l) {
			# Condition for middle page
			$prev_pn = $tw_c - 1
			$next_pn = $tw_c + 1
			$prev_link = '<a href="' + $tw_pre_base + '-' + $prev_pn + '.html">Previous Page</a>'
			if ($prev_pn -eq 1) {
				$prev_link = $prev_link.replace('-1', '')
			}
			$next_link = '<a href="' + $tw_pre_base + '-' + $next_pn + '.html">Next Page</a>'
		} else {
			# Condition for last page
			$prev_pn = $tw_c - 1
			$prev_link = '<a href="' + $tw_pre_base + '-' + $prev_pn + '.html">Previous Page</a>'
			if ($prev_pn -eq 1) {
				$prev_link = $prev_link.replace('-1', '')
			}
			$next_link = ''
		}
		
		$page_links = '<p class="hua-paging">' + $prev_link + ' ' + $next_link + '</p>'
		$footer_i = $footer.replace('<!-- paging -->', $page_links)
		Add-Content $tw_file $footer_i
	}
}

# Account for footer in non-multi-page tagged-with files
foreach ($tagged_with_file in Get-Item(Join-Path -Path $output_dir_tmp -ChildPath 'tagged-with-*')) {
	if (-Not (Select-String -Path $tagged_with_file -pattern 'hua-paging')) {
		$footer_i = $footer.replace('<!-- paging -->', '')
		Add-Content $tagged_with_file $footer_i
	}
}

#
# Post-processing for the master tag list
#

$master_tag_list = ProcessTags $master_tag_list

# Permalink files
foreach ($entry in $entries) {

	# Handle markdown filenames
	if ([System.IO.Path]::GetExtension($entry.File) -eq '.md') {
		
		#Skip if no MarkdownToHTML installed
		if ($no_md -eq 1) { continue }
		$e_fn = [System.IO.Path]::GetFileNameWithoutExtension($entry.File) + '.html'
	} else {
		$e_fn = $entry.'File'
	}

	MasterTags (Join-Path -Path $output_dir_tmp -ChildPath $e_fn) $master_tag_list
}

# Tagged-with files
foreach ($tagged_with_file in Get-Item(Join-Path -Path $output_dir_tmp -ChildPath 'tagged-with-*')) {
	MasterTags $tagged_with_file $master_tag_list 1
}

foreach ($tagged_with_file in Get-Item(Join-Path -Path $output_dir_tmp -ChildPath 'tagged-with-*')) {
	MasterTags $tagged_with_file ''
}

# Handle the index page(s)
$index_base = [io.path]::GetFileNameWithoutExtension($index_file)
$index_pages = Get-ChildItem (Join-Path -Path $output_dir_tmp -ChildPath "$index_base*")
foreach ($index_page in $index_pages) {
	MasterTags $index_page $master_tag_list 1
	MasterTags $index_page ''
}

#
# Compare files generated in temp with those in output directory and replace
#

foreach ($tmp_file in Get-Item(Join-Path -Path $output_dir_tmp -ChildPath '*')) {
	$destination_file = (Join-Path $output_dir -ChildPath (Split-Path $tmp_file -leaf))

	# If the file exists in the output directory, compare
	if (Test-Path (Join-Path $output_dir -ChildPath (Split-Path $tmp_file -leaf))) {
		# Will compare
		if(Compare-Object -ReferenceObject $(Get-Content -Path $destination_file) -DifferenceObject $(Get-Content -Path $tmp_file)) {
			# Replace tagged-with-* file
			Move-Item -Path $tmp_file -Destination $destination_file -Force
		}
	} else {
		# New tagged-with file. Simply move it
		Move-Item -Path $tmp_file -Destination $destination_file -Force
	}
}
Remove-Item $output_dir_tmp -Recurse -Force

#
# Clean up files generated from markdown
#

if ($no_md -ne 1) {
	foreach ($entry in $entries) {

		if ([System.IO.Path]::GetExtension($entry.File) -eq '.md') {
			$e_fn = [System.IO.Path]::GetFileNameWithoutExtension($entry.File) + '.html'

			if (Test-Path (Join-Path -Path $content_dir -ChildPath $e_fn) -PathType Leaf) {
				Remove-Item (Join-Path -Path $content_dir -ChildPath $e_fn)
			} else {
				continue
			}
		}
	}
}

# Removing paging database.
Remove-Item 'paging.csv'

exit 0